import json

import httpx
from mcp import Tool

from .router_types import McpServer
from .nacos_mcp_server_config import NacosMcpServerConfig, ToolSpec


class NacosHttpClient:
    def __init__(self, nacosAddr: str, userName: str, passwd: str) -> None:
        if nacosAddr == "":
            raise ValueError("nacosAddr cannot be an empty string")
        if userName == "":
            raise ValueError("userName cannot be an empty string")
        if passwd == "":
            raise ValueError("passwd cannot be an empty string")

        self.nacosAddr = nacosAddr
        self.userName = userName
        self.passwd = passwd

    def get_mcp_server_by_name(self, name: str) -> McpServer:
        url = "http://{0}/v3/console/ai/mcp?mcpName={1}".format(self.nacosAddr, name)
        headers = {"Content-Type": "application/json", "charset": "utf-8", "userName": self.userName,
                   "password": self.passwd}
        response = httpx.get(url, headers=headers)
        mcp_server = McpServer(name=name, description="", agentConfig={})
        if response.status_code == 200:
            jsonObj = json.loads(response.content.decode("utf-8"))
            data = jsonObj['data']
            config = NacosMcpServerConfig.from_dict(data)
            mcpServer = McpServer(name=config.name, description=config.description if config.description is not None else "",
                                  agentConfig=config.local_server_config)
            mcpServer.mcp_config_detail = config

            if config.protocol != "stdio":
                if len(config.backend_endpoints) > 0:
                    endpoint = config.backend_endpoints[0]
                    url = "http://{0}:{1}{2}".format(endpoint.address, str(
                        endpoint.port), config.remote_server_config.export_path)
                    mcpServer.agentConfig["url"] = url
            return mcpServer
        return mcp_server

    def get_mcp_servers_by_page(self, page_no: int, page_size: int) -> list[McpServer]:
        mcpServers = list[McpServer]()
        try:
            url = "http://{0}/v3/console/ai/mcp/list?pageNo={1}&pageSize={2}".format(self.nacosAddr, str(page_no), str(
                page_size))
            headers = {"Content-Type": "application/json", "charset": "utf-8", "userName": self.userName,
                       "password": self.passwd}
            response = httpx.get(url, headers=headers)
            if response.status_code != 200:
                return []
            jsonObj = json.loads(response.content.decode("utf-8"))
            data = jsonObj['data']
            for mcp_server_dict in data['pageItems']:
                if mcp_server_dict["enabled"]:
                    mcp_name = mcp_server_dict["name"]
                    mcpServer = self.get_mcp_server_by_name(mcp_name)

                    if mcpServer.description == "":
                        continue
                    mcpServers.append(mcpServer)
            return mcpServers
        except Exception as e:
            return mcpServers

    def get_mcp_servers(self) -> list[McpServer]:
        mcpServers = []
        try:
            page_size = 100
            page_no = 1
            url = "http://{0}/v3/console/ai/mcp/list?pageNo={1}&pageSize={2}".format(self.nacosAddr, str(page_no), str(
                page_size))

            headers = {"Content-Type": "application/json", "charset": "utf-8", "userName": self.userName,
                       "password": self.passwd}
            response = httpx.get(url, headers=headers)
            if response.status_code != 200:
                return []
            jsonObj = json.loads(response.content.decode("utf-8"))
            total_count = jsonObj['data']['totalCount']
            total_pages = int(total_count / page_size) + 1

            for i in range(1, total_pages + 1):
                mcps = self.get_mcp_servers_by_page(i, page_size)
                for mcp_server in mcps:
                    mcpServers.append(mcp_server)
            return mcpServers
        except Exception as e:
            return mcpServers

    def update_mcp_tools(self,mcp_name:str, tools: list[Tool]) -> bool:
        url = "http://{0}/v3/console/ai/mcp?mcpName={1}".format(self.nacosAddr, mcp_name)
        headers = {"Content-Type": "application/json", "charset": "utf-8", "userName": self.userName,
                   "password": self.passwd}
        response = httpx.get(url, headers=headers)

        if response.status_code == 200:
            jsonObj = json.loads(response.content.decode("utf-8"))
            data = jsonObj['data']
            tool_list = []
            for tool in tools:
                dct = {}
                dct["name"] = tool.name
                dct["description"] = tool.description
                dct["inputSchema"] = tool.inputSchema
                tool_list.append(dct)
            endpointSpecification = {}
            if data['protocol'] != "stdio":
               endpointSpecification = data['remoteServerConfig']['serviceRef']
               endpointSpecification['type'] = 'REF'
               del data['remoteServerConfig']['serviceRef']

            data['toolSpec']['tools'] = tool_list
            params = {}
            params['mcpName'] = mcp_name
            toolSpecification = data['toolSpec']


            del data['toolSpec']
            del data['backendEndpoints']


            params["serverSpecification"] = json.dumps(data, ensure_ascii=False)
            params["endpointSpecification"] = json.dumps(endpointSpecification, ensure_ascii=False)
            params["toolSpecification"] = json.dumps(toolSpecification, ensure_ascii=False)

            url = "http://" + self.nacosAddr + "/v3/console/ai/mcp?"
            headers = {"Content-Type": "application/x-www-form-urlencoded", "charset": "utf-8", "userName": self.userName,
                       "password": self.passwd}
            response_update = httpx.put(url, headers=headers, data=params)
            if response_update.status_code == 200:
                return True
            else:
                return False
        else:
            return False
